@@ -23,6 +23,7 @@ from yellowdog_client.model import (
     AwsFleetComputeSource,
     AwsFleetPurchaseOption,
     CloudProvider,
+    CreateNamespaceRequest,
     Group,
     ImageOsType,
     InternalUser,
@@ -33,6 +34,7 @@ from yellowdog_client.model import (
     NamespacePolicy,
     NamespaceStorageConfiguration,
     RequirementsAllowance,
+    RoleScope,
     SourceAllowance,
     SourcesAllowance,
     User,
@@ -95,15 +97,19 @@ from yellowdog_cli.utils.settings import (
     PROP_TITLE,
     PROP_TYPE,
     PROP_UNITS,
+    PROP_USERNAME,
     RN_ADD_APPLICATION_REQUEST,
     RN_ADD_GROUP_REQUEST,
     RN_ALLOWANCE,
     RN_APPLICATION,
     RN_CONFIGURED_POOL,
     RN_CREDENTIAL,
+    RN_EXTERNAL_USER,
     RN_GROUP,
     RN_IMAGE_FAMILY,
+    RN_INTERNAL_USER,
     RN_KEYRING,
+    RN_NAMESPACE,
     RN_NAMESPACE_POLICY,
     RN_NUMERIC_ATTRIBUTE_DEFINITION,
     RN_REQUIREMENT_TEMPLATE,
@@ -112,7 +118,6 @@ from yellowdog_cli.utils.settings import (
     RN_STRING_ATTRIBUTE_DEFINITION,
     RN_UPDATE_APPLICATION_REQUEST,
     RN_UPDATE_GROUP_REQUEST,
-    RN_USER,
 )
 from yellowdog_cli.utils.wrapper import ARGS_PARSER, CLIENT, CONFIG_COMMON, main_wrapper
 from yellowdog_cli.utils.ydid_utils import YDIDType, get_ydid_type
@@ -190,8 +195,12 @@ def create_resources(
                 create_group(resource)
             elif resource_type == RN_APPLICATION:
                 create_application(resource)
-            elif resource_type == RN_USER:
-                update_user(resource)
+            elif resource_type == RN_INTERNAL_USER:
+                update_user(resource, internal_user=True)
+            elif resource_type == RN_EXTERNAL_USER:
+                update_user(resource, internal_user=False)
+            elif resource_type == RN_NAMESPACE:
+                create_namespace(resource)
             else:
                 print_error(f"Unknown resource type '{resource_type}'")
         except Exception as e:
@@ -829,67 +838,6 @@ def create_allowance(resource: Dict):
         print(allowance.id)
 
 
-def _get_model_object(class_name: str, resource: Dict, **kwargs):
-    """
-    Return a populated YellowDog model object for the resource.
-    Discard unexpected keywords.
-    """
-
-    def _patch_aws_fleet_enums():
-        if isinstance(model_object, AwsFleetComputeSource):
-            try:
-                model_object.purchaseOption = AwsFleetPurchaseOption[
-                    str(model_object.purchaseOption)
-                ]
-            except KeyError:
-                raise Exception(
-                    "Invalid AWS Fleet Compute Source Purchase Option property: "
-                    f"'{str(model_object.purchaseOption)}'"
-                )
-
-    def _patch_allowance_enums():
-        if (
-            isinstance(model_object, SourceAllowance)
-            or isinstance(model_object, SourcesAllowance)
-            or isinstance(model_object, RequirementsAllowance)
-            or isinstance(model_object, AccountAllowance)
-        ):
-            try:
-                model_object.limitEnforcement = AllowanceLimitEnforcement(
-                    model_object.limitEnforcement
-                )
-                model_object.resetType = AllowanceResetType(model_object.resetType)
-            except KeyError as e:
-                raise Exception(f"Invalid Allowance property: {e}")
-
-    while True:
-        try:
-            model_object = _get_model_class(class_name)(**resource, **kwargs)
-            _patch_aws_fleet_enums()
-            _patch_allowance_enums()
-            return model_object
-        except Exception as e:
-            # Unexpected/missing keyword argument Exception of form:
-            # __init__() got an unexpected keyword argument 'keyword', or
-            # __init__() missing 1 required positional argument: 'credential'
-            if "unexpected" in str(e):
-                keyword = str(e).split("'")[1]
-                print_warning(f"Ignoring unexpected property '{keyword}'")
-                resource.pop(keyword)
-            elif "missing" in str(e):
-                keyword = str(e).split("'")[1]
-                raise Exception(f"Missing expected property '{keyword}'")
-            else:
-                raise e
-
-
-def _get_model_class(class_name: str):
-    """
-    Return a YellowDog model class using its class name.
-    """
-    return getattr(model, class_name)
-
-
 def create_attribute_definition(resource: Dict, resource_type: str):
     """
     Use the API to create/update user attribute definitions.
@@ -1034,7 +982,9 @@ def create_group(resource: Dict):
 
         role_ids_to_add = new_role_ids - current_role_ids
         for role_id in role_ids_to_add:
-            CLIENT.account_client.add_role_to_group(group.id, role_id)
+            CLIENT.account_client.add_role_to_group(
+                group.id, role_id, RoleScope(global_=True)
+            )
             print_log(
                 f"Added Role '{get_role_name_by_id(CLIENT, role_id)}' "
                 f"to Group ({role_id})"
@@ -1176,18 +1126,26 @@ def create_application(resource: Dict):
         update_application(app_id)
 
 
-def update_user(resource: Dict):
+def update_user(resource: Dict, internal_user: bool):
     """
-    Update a user (specified by name or ID). Will also add or remove
+    Update a user specified by name, username or ID. Will also add or remove
     groups specified by their names or IDs.
     """
     name = resource.get(PROP_NAME)
+    username = resource.get(PROP_USERNAME)
     id = resource.get(PROP_ID)
 
-    if name is None and id is None:
+    # Check we have a user identity
+    if internal_user:
+        if not any([username, name, id]):
+            raise Exception(
+                f"Expected one of '{PROP_NAME}', '{PROP_USERNAME}', '{PROP_ID}' "
+                f"to be defined for resource '{RN_INTERNAL_USER}' ({resource})"
+            )
+    elif not any([name, id]):
         raise Exception(
             f"Expected one of '{PROP_NAME}', '{PROP_ID}' to be defined for "
-            f"resource '{RN_USER}'"
+            f"resource '{RN_EXTERNAL_USER}' ({resource})"
         )
 
     groups: Optional[List[str]] = resource.pop(PROP_GROUPS, None)
@@ -1234,19 +1192,21 @@ def update_user(resource: Dict):
                 f"({group_id})"
             )
 
-    # Main logic: try name then ID if present; check for ID match
-    user = None
+    # Main logic: try name, username, then ID if present; check for ID match
+    user: Optional[User] = None
     if name is not None:
         user = get_user_by_name_or_id(CLIENT, name)
+    if user is None and username is not None:
+        user = get_user_by_name_or_id(CLIENT, username)
     if user is not None and id is not None:
         if user.id != id:
-            raise Exception(f"User name '{name}' and supplied ID do not match")
+            raise Exception(f"User name and supplied ID do not match ({resource})")
     if user is None and id is not None:
         user = get_user_by_name_or_id(CLIENT, id)
 
     if user is None:
         print_warning(
-            f"User '{name}' not found; Users cannot be created using "
+            f"User not found ({resource}); Users cannot be created using "
             "the CLI, please use the YellowDog Portal"
         )
         return
@@ -1259,6 +1219,91 @@ def update_user(resource: Dict):
         print_log(f"Nothing to do for User '{username}' ({user.id})")
 
 
+def create_namespace(resource: Dict):
+    """
+    Create a namespace.
+    """
+    try:
+        name = resource[PROP_NAME]
+    except KeyError as e:
+        raise Exception(f"Expected property to be defined ({e})")
+
+    try:
+        namespace_id = CLIENT.namespaces_client.create_namespace(
+            CreateNamespaceRequest(namespace=name)
+        )
+    except Exception as e:
+        if "ConflictException" in str(e):
+            raise Exception(f"Namespace '{name}' already exists")
+        raise Exception(f"Failed to create namespace '{name}' ({e})")
+
+    print_log(f"Created namespace '{name}' ({namespace_id})")
+
+    if ARGS_PARSER.quiet:
+        print(namespace_id)
+
+
+def _get_model_object(class_name: str, resource: Dict, **kwargs):
+    """
+    Return a populated YellowDog model object for the resource.
+    Discard unexpected keywords.
+    """
+
+    def _patch_aws_fleet_enums():
+        if isinstance(model_object, AwsFleetComputeSource):
+            try:
+                model_object.purchaseOption = AwsFleetPurchaseOption[
+                    str(model_object.purchaseOption)
+                ]
+            except KeyError:
+                raise Exception(
+                    "Invalid AWS Fleet Compute Source Purchase Option property: "
+                    f"'{str(model_object.purchaseOption)}'"
+                )
+
+    def _patch_allowance_enums():
+        if (
+            isinstance(model_object, SourceAllowance)
+            or isinstance(model_object, SourcesAllowance)
+            or isinstance(model_object, RequirementsAllowance)
+            or isinstance(model_object, AccountAllowance)
+        ):
+            try:
+                model_object.limitEnforcement = AllowanceLimitEnforcement(
+                    model_object.limitEnforcement
+                )
+                model_object.resetType = AllowanceResetType(model_object.resetType)
+            except KeyError as e:
+                raise Exception(f"Invalid Allowance property: {e}")
+
+    while True:
+        try:
+            model_object = _get_model_class(class_name)(**resource, **kwargs)
+            _patch_aws_fleet_enums()
+            _patch_allowance_enums()
+            return model_object
+        except Exception as e:
+            # Unexpected/missing keyword argument Exception of form:
+            # __init__() got an unexpected keyword argument 'keyword', or
+            # __init__() missing 1 required positional argument: 'credential'
+            if "unexpected" in str(e):
+                keyword = str(e).split("'")[1]
+                print_warning(f"Ignoring unexpected property '{keyword}'")
+                resource.pop(keyword)
+            elif "missing" in str(e):
+                keyword = str(e).split("'")[1]
+                raise Exception(f"Missing expected property '{keyword}'")
+            else:
+                raise e
+
+
+def _get_model_class(class_name: str):
+    """
+    Return a YellowDog model class using its class name.
+    """
+    return getattr(model, class_name)
+
+
 def _create_image_family(
     image_family: MachineImageFamily, fq_name: str
 ) -> MachineImageFamily:
