@@ -27,8 +27,10 @@ from yellowdog_client.model import (
     KeyringSummary,
     MachineImageFamilySearch,
     MachineImageFamilySummary,
+    Namespace,
     NamespacePolicy,
     NamespacePolicySearch,
+    NamespaceSearch,
     NamespaceStorageConfiguration,
     Node,
     NodeSearch,
@@ -82,6 +84,7 @@ from yellowdog_cli.utils.settings import (
     RN_GROUP,
     RN_IMAGE_FAMILY,
     RN_KEYRING,
+    RN_NAMESPACE,
     RN_NUMERIC_ATTRIBUTE_DEFINITION,
     RN_REQUIREMENT_TEMPLATE,
     RN_ROLE,
@@ -141,6 +144,8 @@ def main():
         list_allowances()
     elif ARGS_PARSER.attribute_definitions:
         list_attribute_definitions()
+    elif ARGS_PARSER.namespaces:
+        list_namespaces()
     elif ARGS_PARSER.namespace_policies:
         list_namespace_policies()
     elif ARGS_PARSER.users:
@@ -169,6 +174,7 @@ def check_for_valid_option() -> bool:
         ARGS_PARSER.image_families,
         ARGS_PARSER.instances,
         ARGS_PARSER.keyrings,
+        ARGS_PARSER.namespaces,
         ARGS_PARSER.namespace_policies,
         ARGS_PARSER.namespace_storage_configurations,
         ARGS_PARSER.nodes,
@@ -388,7 +394,7 @@ def list_compute_requirements():
         print_log("Listing active Compute Requirements only")
 
     cr_search = ComputeRequirementSearch(
-        namespace=CONFIG_COMMON.namespace,
+        namespace=None if CONFIG_COMMON.namespace == "" else CONFIG_COMMON.namespace,
     )
     search_client: SearchClient = CLIENT.compute_client.get_compute_requirements(
         cr_search
@@ -691,7 +697,9 @@ def list_image_families():
     """
     image_search = MachineImageFamilySearch(
         includePublic=True,
-        namespace=CONFIG_COMMON.namespace,  # Supports partial match
+        namespace=(
+            None if CONFIG_COMMON.namespace == "" else CONFIG_COMMON.namespace
+        ),  # Supports partial match
         familyName=CONFIG_COMMON.name_tag,  # Supports partial match
     )
     search_client: SearchClient = CLIENT.images_client.get_image_families(image_search)
@@ -867,6 +875,31 @@ def list_attribute_definitions():
     print_yd_object_list(attribute_definition_list)
 
 
+def list_namespaces():
+    """
+    List namespaces.
+    """
+
+    namespaces: List[Namespace] = CLIENT.namespaces_client.get_namespaces(
+        NamespaceSearch()
+    ).list_all()
+
+    if len(namespaces) == 0:
+        print_log("No Namespaces found")
+        return
+
+    if not ARGS_PARSER.details:
+        print_numbered_object_list(CLIENT, namespaces)
+        return
+
+    print_yd_object_list(
+        [
+            (namespace, {PROP_RESOURCE: RN_NAMESPACE})
+            for namespace in select(CLIENT, namespaces)
+        ]
+    )
+
+
 def list_namespace_policies():
     """
     List namespace policies.
