@@ -3,7 +3,7 @@ Various utility functions for finding objects, etc.
 """
 
 from functools import lru_cache
-from typing import List, Optional
+from typing import Callable, List, Optional, Tuple
 
 from yellowdog_client import PlatformClient
 from yellowdog_client.common import SearchClient
@@ -15,8 +15,10 @@ from yellowdog_client.model import (
     ComputeRequirementSearch,
     ComputeRequirementStatus,
     ComputeRequirementTemplate,
+    ComputeRequirementTemplateSearch,
     ComputeRequirementTemplateSummary,
     ComputeSourceTemplate,
+    ComputeSourceTemplateSearch,
     ComputeSourceTemplateSummary,
     ExternalUser,
     GroupSearch,
@@ -37,6 +39,7 @@ from yellowdog_client.model import (
     User,
     UserSearch,
     WorkerPool,
+    WorkerPoolSearch,
     WorkerPoolSummary,
     WorkRequirementStatus,
     WorkRequirementSummary,
@@ -176,14 +179,14 @@ def get_work_requirement_summary_by_name_or_id(
 
 
 def _find_id_by_name(
-    name: str, client: PlatformClient, find_function: callable
+    name: str, client: PlatformClient, find_function: Callable
 ) -> Optional[str]:
     """
     Generic function to find the ID of an entity by namespace and name.
     """
     namespace, name = split_namespace_and_name(name)
 
-    entities = find_function(client)
+    entities = find_function(client, namespace)
 
     exact_matching_entities = []
     inexact_matching_entities = []
@@ -228,12 +231,18 @@ def find_compute_source_template_id_by_name(
 
 @lru_cache
 def get_all_compute_source_templates(
-    client: PlatformClient,
+    client: PlatformClient, namespace: Optional[str]
 ) -> List[ComputeSourceTemplateSummary]:
     """
-    Cache the list of Sources.
+    Cache the list of Compute Source Templates.
     """
-    return client.compute_client.find_all_compute_source_templates()
+    cst_search = ComputeSourceTemplateSearch(
+        namespaces=None if namespace in [None, ""] else [namespace]
+    )
+    cst_search_client: SearchClient = (
+        client.compute_client.get_compute_source_templates(cst_search)
+    )
+    return cst_search_client.list_all()
 
 
 def clear_compute_source_template_cache():
@@ -255,12 +264,18 @@ def find_compute_requirement_template_id_by_name(
 
 @lru_cache
 def get_all_compute_requirement_templates(
-    client: PlatformClient,
+    client: PlatformClient, namespace: Optional[str]
 ) -> List[ComputeRequirementTemplateSummary]:
     """
     Cache the list of Compute Requirement Templates.
     """
-    return client.compute_client.find_all_compute_requirement_templates()
+    crt_search = ComputeRequirementTemplateSearch(
+        namespaces=None if namespace in [None, ""] else [namespace]
+    )
+    crt_search_client: SearchClient = (
+        client.compute_client.get_compute_requirement_templates(crt_search)
+    )
+    return crt_search_client.list_all()
 
 
 def clear_compute_requirement_template_cache():
@@ -283,11 +298,19 @@ def get_compute_requirement_id_by_worker_pool_id(
         return worker_pool.computeRequirementId
 
 
-def get_all_worker_pools(client: PlatformClient) -> List[WorkerPoolSummary]:
+def get_all_worker_pools(
+    client: PlatformClient, namespace: Optional[str]
+) -> List[WorkerPoolSummary]:
     """
-    Return all Worker Pool summaries.
+    Return all Worker Pool summaries for a namespace.
     """
-    return client.worker_pool_client.find_all_worker_pools()
+    wp_search = WorkerPoolSearch(
+        namespaces=None if namespace in [None, ""] else [namespace]
+    )
+    wp_search_client: SearchClient = client.worker_pool_client.get_worker_pools(
+        wp_search
+    )
+    return wp_search_client.list_all()
 
 
 @lru_cache
@@ -299,6 +322,12 @@ def find_image_family_reference_by_name(
     Fully qualified name is used for non-ambiguous PRIVATE image families.
     """
 
+    if image_family_name.startswith("ami-") or image_family_name.startswith(
+        "ocid1.image."
+    ):
+        # We can identify AWS AMIs and OCI OCIDs, and return immediately
+        return image_family_name
+
     original_image_family_name = image_family_name
 
     # Remove leading 'yd/' prefix if necessary
@@ -310,10 +339,25 @@ def find_image_family_reference_by_name(
     namespace, name = split_namespace_and_name(image_family_name)
 
     if_search = MachineImageFamilySearch(
-        familyName=name, namespace=namespace, includePublic=True
+        familyName=name,
+        namespaces=None if namespace in [None, ""] else [namespace],
+        includePublic=True,
     )
     search_client: SearchClient = client.images_client.get_image_families(if_search)
-    image_families: List[MachineImageFamilySummary] = search_client.list_all()
+
+    try:
+        image_families: List[MachineImageFamilySummary] = search_client.list_all()
+    except Exception as e:
+        if "MissingPermissionException" in str(e) and "IMAGE_READ" in str(e):
+            # IMAGE_READ permission (globally, or for this namespace) is absent;
+            # abandon lookup and return the original image family name
+            print_log(
+                f"Note: failed to resolve image name '{original_image_family_name}' "
+                "due to lack of 'IMAGE_READ' permission; using original name"
+            )
+            return original_image_family_name
+        else:
+            raise
 
     # Partial names will match, so filter for exact matches only
     image_families = [
@@ -484,7 +528,7 @@ def get_non_exact_namespace_matches(
     return matching_namespaces
 
 
-def split_namespace_and_name(reference: str) -> (Optional[str], str):
+def split_namespace_and_name(reference: str) -> Tuple[Optional[str], str]:
     """
     Split a name into an (optional) namespace and a name.
     """
